"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[5053],{657:function(t,n,c){c.d(n,{Z:function(){return d}});var e=c(6252),r=c(3577);const a={class:"icon-container"},l=["src","alt"];function s(t,n,c,s,o,i){return(0,e.wg)(),(0,e.iD)("div",a,[c.url?.length?((0,e.wg)(),(0,e.iD)("img",{key:0,class:"icon",src:c.url,alt:c.alt},null,8,l)):i.className?.length?((0,e.wg)(),(0,e.iD)("i",{key:1,class:(0,r.C_)(["icon",i.className]),style:(0,r.j5)({color:c.color})},null,6)):(0,e.kq)("",!0)])}var o={props:{class:{type:String},url:{type:String},color:{type:String,default:""},alt:{type:String,default:""}},computed:{className(){return this.class}}},i=c(3744);const u=(0,i.Z)(o,[["render",s],["__scopeId","data-v-706a3bd1"]]);var d=u},4221:function(t,n,c){c.r(n),c.d(n,{default:function(){return p}});var e=c(6252);const r=t=>((0,e.dD)("data-v-7c29571d"),t=t(),(0,e.Cn)(),t),a={class:"row plugin execute-container"},l=r((()=>(0,e._)("h1",null,"Execute Action",-1)));function s(t,n,c,r,s,o){const i=(0,e.up)("ActionEditor");return(0,e.wg)(),(0,e.iD)("div",a,[(0,e._)("main",null,[l,(0,e.Wm)(i)])])}var o=c(9502),i={components:{ActionEditor:o.Z}},u=c(3744);const d=(0,u.Z)(i,[["render",s],["__scopeId","data-v-7c29571d"]]);var p=d}}]);
//# sourceMappingURL=5053.d4332348.js.map