"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[2305],{2305:function(e,n,t){t.r(n),t.d(n,{default:function(){return i}});var u=t(6252);function a(e,n,t,a,r,c){const s=(0,u.up)("Media");return(0,u.wg)(),(0,u.j4)(s,{"plugin-name":"media.chromecast"})}var r=t(9339),c={components:{Media:r["default"]}},s=t(3744);const p=(0,s.Z)(c,[["render",a]]);var i=p}}]);
//# sourceMappingURL=2305.ea3c4ae0.js.map