"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[8697],{657:function(t,n,a){a.d(n,{Z:function(){return d}});var s=a(6252),o=a(3577);const e={class:"icon-container"},r=["src","alt"];function l(t,n,a,l,c,i){return(0,s.wg)(),(0,s.iD)("div",e,[a.url?.length?((0,s.wg)(),(0,s.iD)("img",{key:0,class:"icon",src:a.url,alt:a.alt},null,8,r)):i.className?.length?((0,s.wg)(),(0,s.iD)("i",{key:1,class:(0,o.C_)(["icon",i.className]),style:(0,o.j5)({color:a.color})},null,6)):(0,s.kq)("",!0)])}var c={props:{class:{type:String},url:{type:String},color:{type:String,default:""},alt:{type:String,default:""}},computed:{className(){return this.class}}},i=a(3744);const u=(0,i.Z)(c,[["render",l],["__scopeId","data-v-706a3bd1"]]);var d=u},2717:function(t,n,a){a.d(n,{Z:function(){return f}});var s=a(6252);const o=t=>((0,s.dD)("data-v-2edff8b7"),t=t(),(0,s.Cn)(),t),e={class:"restart-btn-container"},r=o((()=>(0,s._)("i",{class:"fas fa-redo-alt"},null,-1)));function l(t,n,a,o,l,c){const i=(0,s.up)("ConfirmDialog");return(0,s.wg)(),(0,s.iD)("div",e,[(0,s.Wm)(i,{ref:"modal",onInput:c.restart},{default:(0,s.w5)((()=>[(0,s.Uk)(" Are you sure that you want to restart the application? ")])),_:1},8,["onInput"]),(0,s._)("button",{class:"btn btn-default restart-btn",onClick:n[0]||(n[0]=(...t)=>c.showDialog&&c.showDialog(...t)),onTouch:n[1]||(n[1]=(...t)=>c.showDialog&&c.showDialog(...t))},[r,(0,s.Uk)("   Restart Application ")],32)])}var c=a(3513),i=a(8637),u={name:"RestartButton",components:{ConfirmDialog:c.Z},mixins:[i.Z],methods:{showDialog(){this.$refs.modal.show()},async restart(){await this.request("application.restart")}}},d=a(3744);const p=(0,d.Z)(u,[["render",l],["__scopeId","data-v-2edff8b7"]]);var f=p},8735:function(t,n,a){a.d(n,{Z:function(){return d}});var s=a(6252),o=a(3577);const e={key:0,class:"icon"};function r(t,n,a,r,l,c){const i=(0,s.up)("Icon");return(0,s.wg)(),(0,s.iD)("div",{class:(0,o.C_)(["tab",a.selected?"selected":""]),onClick:n[0]||(n[0]=n=>t.$emit("input"))},[a.iconClass?.length||a.iconUrl?.length?((0,s.wg)(),(0,s.iD)("span",e,[(0,s.Wm)(i,{class:(0,o.C_)(a.iconClass),url:a.iconUrl},null,8,["class","url"])])):(0,s.kq)("",!0),(0,s.Uk)("   "),(0,s.WI)(t.$slots,"default",{},void 0,!0)],2)}var l=a(657),c={name:"Tab",components:{Icon:l.Z},emits:["input"],props:{selected:{type:Boolean,default:!1},iconClass:{type:String},iconUrl:{type:String}}},i=a(3744);const u=(0,i.Z)(c,[["render",r],["__scopeId","data-v-f3217d34"]]);var d=u},3176:function(t,n,a){a.d(n,{Z:function(){return i}});var s=a(6252);const o={class:"tabs"};function e(t,n,a,e,r,l){return(0,s.wg)(),(0,s.iD)("div",o,[(0,s.WI)(t.$slots,"default",{},void 0,!0)])}var r={name:"Tabs"},l=a(3744);const c=(0,l.Z)(r,[["render",e],["__scopeId","data-v-f4300bb0"]]);var i=c}}]);
//# sourceMappingURL=8697.3f9798ec.js.map