"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[6640],{6640:function(l,a,e){e.r(a),e.d(a,{default:function(){return d}});var t=e(8637),n={mixins:[t.Z],emits:["add-to-playlist","back","create-playlist","download","download-audio","path-change","play","remove-from-playlist","remove-playlist","rename-playlist"],props:{filter:{type:String,default:""},loading:{type:Boolean,default:!1},mediaPlugin:{type:String},selectedPlaylist:{default:null},selectedChannel:{default:null}},data(){return{loading_:!1}},computed:{isLoading(){return this.loading||this.loading_}}};const i=n;var d=i}}]);
//# sourceMappingURL=6640.9ad3c551.js.map