"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[1652],{3222:function(e,t,a){a.d(t,{Z:function(){return u}});var s=a(6252),l=a(3577);const i={class:"no-items-container"};function n(e,t,a,n,o,d){return(0,s.wg)(),(0,s.iD)("div",i,[(0,s._)("div",{class:(0,l.C_)(["no-items fade-in",{shadow:a.withShadow}])},[(0,s.WI)(e.$slots,"default",{},void 0,!0)],2)])}var o={name:"NoItems",props:{withShadow:{type:Boolean,default:!0}}},d=a(3744);const r=(0,d.Z)(o,[["render",n],["__scopeId","data-v-4856c4d7"]]);var u=r},1652:function(e,t,a){a.r(t),a.d(t,{default:function(){return z}});var s=a(6252),l=a(3577);const i={class:"media-youtube-playlist"},n={key:1,class:"playlist-container"},o={class:"header"},d={class:"banner"},r=["src"],u={class:"row info-container"},c={class:"info"},m={class:"row"},p=["href"],h={key:1,class:"title"},y={class:"n-items"},w={key:0,class:"row"},f={class:"description"},g={key:1,class:"row"},v={class:"channel"},_=["href"];function k(e,t,a,k,b,I){const D=(0,s.up)("Loading"),$=(0,s.up)("NoItems"),Z=(0,s.up)("Results");return(0,s.wg)(),(0,s.iD)("div",i,[b.loading?((0,s.wg)(),(0,s.j4)(D,{key:0})):((0,s.wg)(),(0,s.iD)("div",n,[(0,s._)("div",o,[(0,s._)("div",d,[a.metadata?.image?.length?((0,s.wg)(),(0,s.iD)("img",{key:0,src:a.metadata?.image},null,8,r)):(0,s.kq)("",!0)]),(0,s._)("div",u,[(0,s._)("div",c,[(0,s._)("div",m,[a.metadata?.url?((0,s.wg)(),(0,s.iD)("a",{key:0,class:"title",href:a.metadata?.url,target:"_blank",rel:"noopener noreferrer"},(0,l.zw)(I.name),9,p)):((0,s.wg)(),(0,s.iD)("span",h,(0,l.zw)(I.name),1)),(0,s._)("div",y,(0,l.zw)(I.nItems)+" videos",1)]),a.metadata?.description?((0,s.wg)(),(0,s.iD)("div",w,[(0,s._)("div",f,(0,l.zw)(a.metadata?.description),1)])):(0,s.kq)("",!0),a.metadata?.channel_url?((0,s.wg)(),(0,s.iD)("div",g,[(0,s._)("div",v,[(0,s.Uk)(" Uploaded by "),(0,s._)("a",{href:a.metadata.channel_url,target:"_blank",rel:"noopener noreferrer"},(0,l.zw)(a.metadata?.channel),9,_)])])):(0,s.kq)("",!0)])])]),I.nItems?((0,s.wg)(),(0,s.j4)(Z,{key:1,results:b.items,sources:{youtube:!0},filter:a.filter,playlist:a.id,"selected-result":b.selectedResult,onAddToPlaylist:t[0]||(t[0]=t=>e.$emit("add-to-playlist",t)),onDownload:t[1]||(t[1]=t=>e.$emit("download",t)),onDownloadAudio:t[2]||(t[2]=t=>e.$emit("download-audio",t)),onOpenChannel:t[3]||(t[3]=t=>e.$emit("open-channel",t)),onPlay:t[4]||(t[4]=t=>e.$emit("play",t)),onPlayWithOpts:t[5]||(t[5]=t=>e.$emit("play-with-opts",t)),onRemoveFromPlaylist:t[6]||(t[6]=t=>e.$emit("remove-from-playlist",t)),onSelect:t[7]||(t[7]=e=>b.selectedResult=e)},null,8,["results","filter","playlist","selected-result"])):((0,s.wg)(),(0,s.j4)($,{key:0,"with-shadow":!1},{default:(0,s.w5)((()=>[(0,s.Uk)(" No videos found. ")])),_:1}))]))])}var b=a(3222),I=a(6791),D=a(1602),$=a(8637),Z={mixins:[$.Z],emits:["add-to-playlist","download","download-audio","open-channel","play","play-with-opts","remove-from-playlist"],components:{Loading:I.Z,NoItems:b.Z,Results:D.Z},props:{id:{type:String,required:!0},filter:{type:String,default:null},metadata:{type:Object,default:null}},data(){return{items:[],loading:!1,selectedResult:null}},computed:{name(){return this.metadata?.title||this.metadata?.name},nItems(){return this.metadata?.videos||this.items?.length||0}},methods:{async loadItems(){this.loading=!0;try{this.items=(await this.request("youtube.get_playlist",{id:this.id})).map((e=>({...e,type:"youtube"})))}finally{this.loading=!1}}},mounted(){this.setUrlArgs({playlist:this.id}),this.loadItems()},unmounted(){this.setUrlArgs({playlist:null})}},R=a(3744);const q=(0,R.Z)(Z,[["render",k],["__scopeId","data-v-7f3997be"]]);var z=q}}]);
//# sourceMappingURL=1652.7d49cd50.js.map