"use strict";(self["webpackChunkplatypush"]=self["webpackChunkplatypush"]||[]).push([[7381],{7381:function(a,e,r){r.r(e),r.d(e,{default:function(){return o}});var t=r(6252);function s(a,e,r,s,n,c){const u=(0,t.up)("Camera");return(0,t.wg)(),(0,t.j4)(u,{"camera-plugin":"ir.mlx90640",ref:"camera"},null,512)}var n=r(7528),c={name:"CameraIrMlx90640",components:{Camera:n["default"]},mounted(){const a=this.$root.config[`camera.${this.cameraPlugin}`]||{};a.resolution||(this.$refs.camera.attrs.resolution=[32,24]),a.scale_x||(this.$refs.camera.attrs.scale_x=15),a.scale_y||(this.$refs.camera.attrs.scale_y=15)}},u=r(3744);const l=(0,u.Z)(c,[["render",s]]);var o=l}}]);
//# sourceMappingURL=7381.bf8f1b35.js.map