# Supported audio extensions
audio_extensions = {
    '3gp',
    'aa',
    'aac',
    'aax',
    'act',
    'aiff',
    'amr',
    'ape',
    'au',
    'awb',
    'dct',
    'dss',
    'dvf',
    'flac',
    'gsm',
    'iklax',
    'ivs',
    'm4a',
    'm4b',
    'm4p',
    'mmf',
    'mp3',
    'mpc',
    'msv',
    'nmf',
    'nsf',
    'ogg,',
    'opus',
    'ra,',
    'raw',
    'sln',
    'tta',
    'vox',
    'wav',
    'wma',
    'wv',
    'webm',
    '8svx',
}


# Supported video extensions
video_extensions = {
    'webm',
    'mkv',
    'flv',
    'vob',
    'ogv',
    'ogg',
    'drc',
    'gif',
    'gifv',
    'mng',
    'avi',
    'mts',
    'm2ts',
    'mov',
    'qt',
    'wmv',
    'yuv',
    'rm',
    'rmvb',
    'asf',
    'amv',
    'mp4',
    'm4p',
    'm4v',
    'mpg',
    'mp2',
    'mpeg',
    'mpe',
    'mpv',
    'm2v',
    'svi',
    '3gp',
    '3g2',
    'mxf',
    'roq',
    'nsv',
    'f4v',
    'f4p',
    'f4a',
    'f4b',
}


# vim:sw=4:ts=4:et:
